package com.capitalware.ufmsm;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;

import javax.swing.*;
import javax.swing.table.*;

import org.jdom.JDOMException;

import com.capitalware.ufmsm.models.SortFilterModel;
import com.capitalware.ufmsm.models.StatusQueueModel;
import com.capitalware.ufmsm.mq.MQGetOneMessage;
import com.capitalware.ufmsm.mq.MQGetStatusMessages;
import com.capitalware.ufmsm.mq.objects.MQObject;
import com.capitalware.ufmsm.objects.CWConstants;
import com.capitalware.ufmsm.popups.AboutBox;
import com.capitalware.ufmsm.popups.WorkflowDialog;
import com.capitalware.ufmsm.renders.StatusQRender;
import com.capitalware.ufmsm.xml.MQXml;

/**
 * UFM Status Monitor will display a JFrame listing the current channel status.
 * This class is the view ("V") of the MVC.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class UFMSM extends JFrame implements ActionListener, ComponentListener
{
   private JScrollPane           jSp = null;
   private MQGetStatusMessages   mqgsm = null;
   private MQGetOneMessage       mqgom = null;   
   private StatusQueueModel      sqm = null;
   private SortFilterModel       sorter = null;
   private JTable                table = null;
   private MQObject              mqo = null;

   /**
    * Mainline - entry point
    * @param args
    */
   public static void main(String args[])
   {
      if (args.length == 0)
         new UFMSM("mq.xml");
      else
         new UFMSM(args[0]);
   }

   /**
    * The constructor
    *
    * @param instance
    */
   public UFMSM(String mqXmlFile)
   {
      super();
      
      //Turn off metal's use of bold fonts
      UIManager.put("swing.boldMetal", Boolean.FALSE);
      
      addComponentListener(this);

      setSize(CWConstants.MAIN_MIN_WIDTH, CWConstants.MAIN_MIN_HEIGHT);     // w * h
      getContentPane().setLayout(new GridBagLayout());

      addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent we)
         {
            dismiss();
         }
      });

      if ( !(System.getProperty("os.name").toLowerCase().startsWith("linux")) )
      {
         try
         {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
         } catch (Exception ex) {}
      }

      // set up menus
      menuBar();

      setTitle(CWConstants.PRODUCT_NAME + " v" + CWConstants.VERSION);
      ImageIcon icon = new ImageIcon(UFMSM.class.getResource("gif/ufm32.gif"));
      this.setIconImage(icon.getImage());

      this.setVisible(true);
      validate();

      
      try
      {
         MQXml mqp = new MQXml();
         mqp.readXMLFile(mqXmlFile);
         mqo = mqp.getMQObject();
         mqo.checkQM();
         
         /* This class will handle retrieving 1 message when the user clicks on it */
         mqgom = new MQGetOneMessage(mqo, UFMSM.this);
         mqgom.connectAndOpenQ();
         
         /* This class (thread) will constantly update the main GUI */
         mqgsm = new MQGetStatusMessages(mqo, UFMSM.this);
         mqgsm.start();
      }
      catch (IOException e)
      {
         JOptionPane.showMessageDialog( this,
                                       e.getLocalizedMessage(),
                                       "Error: IOException",
                                       JOptionPane.ERROR_MESSAGE);
         dismiss();
      }
      catch (JDOMException e)
      {
         JOptionPane.showMessageDialog( this,
                                         e.getLocalizedMessage(),
                                         "Error: JDOMException",
                                         JOptionPane.ERROR_MESSAGE);
         dismiss();
      }
      catch (IllegalArgumentException e)
      {
         JOptionPane.showMessageDialog( this,
                                         e.getLocalizedMessage(),
                                         "Error: IllegalArgumentException",
                                         JOptionPane.ERROR_MESSAGE);
         dismiss();
      }
   }
   
   /**
    * Add a component to main pane.
    * @param c
    * @param gbc
    * @param x
    * @param y
    * @param w
    * @param h
    */
   private void add(Component c, GridBagConstraints gbc2, int x, int y, int w, int h)
   {
      gbc2.gridx = x;
      gbc2.gridy = y;
      gbc2.gridwidth = w;
      gbc2.gridheight = h;
      getContentPane().add(c, gbc2);
   }
   
   /**
    * Create a menu bar for the application.
    */
   private void menuBar( )
   {
      // create menu - File, Edit, View & Help
      JMenuBar menuBar = new JMenuBar();
      setJMenuBar(menuBar);
      
      JMenu fileMenu = new JMenu("File");
      menuBar.add(fileMenu);

      JMenuItem exitItem = new JMenuItem("Exit");
      exitItem.addActionListener(this);
      fileMenu.add(exitItem);

      JMenu HelpMenu = new JMenu("Help");
      menuBar.add(HelpMenu);

      JMenuItem aboutItem = new JMenuItem("About");
      aboutItem.addActionListener(this);
      HelpMenu.add(aboutItem);
   }

   /**
    * Size out the columns of the JTable.
   */
   private void sizeOutColumns()
   {
      TableColumnModel columnModel = table.getColumnModel();
      TableColumn column;

      /*
       * Rssize the columns
       */
      for (int i=0; i < table.getColumnCount(); i++)
      {
         column = columnModel.getColumn(i);
         
         StatusQRender gr = new StatusQRender();
         gr.setHorizontalAlignment(SwingConstants.CENTER);
         column.setCellRenderer(gr);

         // Do not set the size for the last column
         if (i < table.getColumnCount() - 1)
         {
            column.setPreferredWidth(CWConstants.statusQcolumnWidth[i]);
            column.setMaxWidth(CWConstants.statusQcolumnWidth[i]);
         }
      }
   }

   /**
    * actionPerformed is called by the JVM Gui handler
    * @param evt
    */
   public void actionPerformed(ActionEvent evt)
   {
      String  arg = evt.getActionCommand();

      if (arg.equals("About"))
      {
         AboutBox about = new AboutBox( this,
                                        CWConstants.PRODUCT_NAME,
                                        CWConstants.VERSION,
                                        CWConstants.RELEASE_DATE,
                                        CWConstants.COPYRIGHT_DATE,
                                        "gif/ufm32.gif");
         about.show();
      }
      else if (arg.equals("Exit"))
      {
         dismiss();
      }
      else
      {
         System.out.println( arg + " - NOT defined!!!" );
      }
   }

   /**
    * Update the table data with new data.
    * @param al
    */
   public void updateTable(ArrayList al)
   {
      SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");

      if (al.size() > 0)
      {
         if (sqm == null)
         {
            sqm = new StatusQueueModel(al);
            sorter = new SortFilterModel(sqm);
            sorter.sort(0);

            // show table
            table = new JTable(sorter);
            table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            // Do not allow the User to Move a Column in a JTable Component
            table.getTableHeader().setReorderingAllowed(false);
            jSp = new JScrollPane(table);
   
            /* Allow the user to sort it. */
            sorter.addMouseListener(table);

            table.addMouseListener(new MouseAdapter()
            {
               public void mouseClicked(MouseEvent evt)
               {
                  // check for double click
                  if (evt.getClickCount() < 2) return;

                  int rr = table.getSelectedRow();
                  
                  if (rr >= 0)
                  {
                     Object[] xx = sorter.getRowData(rr);
                     
                     byte[] msgData = mqgom.getMessage((byte[])xx[CWConstants.statusQcolumns - 1]);
                     
                     if (msgData != null)
                     {
                        WorkflowDialog wfd = new WorkflowDialog(UFMSM.this, sorter.getValueAt(rr, 0)+" : "+sorter.getValueAt(rr, 1)+" : "+sorter.getValueAt(rr, 7));
                        wfd.setMessageData(msgData);
                        wfd.show();
                     }
                  }
               }
            });
            
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.anchor = GridBagConstraints.CENTER;
            gbc.fill = GridBagConstraints.BOTH;
            gbc.weightx = 1;
            gbc.weighty = 1;
            gbc.insets = new Insets(0,0,0,0);

            add(jSp, gbc, 0, 0, 1, 1);  // c , r, cw, rh
   
            ((DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer()).setHorizontalAlignment(SwingConstants.CENTER);
   
            sizeOutColumns( );
   
            // force the pane to be updated.
            this.setVisible(true);
            repaint();
         }
         else
         {
            int newDataLen = al.size();
            for (int i=0; i < newDataLen;i++)
               sqm.addUniqueDataToTable((Object[])al.get(i));
            
            sorter.updatedSortFilter();
            sorter.sort(0);
   
            SwingUtilities.invokeLater(new Runnable()
            {
               public void run()
               {
                  sorter.fireTableDataChanged();
               }
            });
         }
      }

      setTitle(CWConstants.PRODUCT_NAME + " : " + mqo.qMgrName + " : " + mqo.qObject.qName + " : Last update at " + timeFormat.format(new java.util.Date()));
   }

   /**
    * The user is resizing the component
    * @param e
    */
   public void componentResized(ComponentEvent e)
   {
      boolean resize = false;
      int width = getWidth();
      int height = getHeight();

      // Check if either the width or the height are below minimum

      if (width < CWConstants.MAIN_MIN_WIDTH)
      {
         resize = true;
         width = CWConstants.MAIN_MIN_WIDTH;
      }

      if (height < CWConstants.MAIN_MIN_HEIGHT)
      {
         resize = true;
         height = CWConstants.MAIN_MIN_HEIGHT;
      }

      if (resize)
         setSize(width, height);
   }

   public void componentMoved(ComponentEvent e)
   {}

   public void componentShown(ComponentEvent e)
   {}

   public void componentHidden(ComponentEvent e)
   {}

   /**
    * Time to cleanup the JFrame
    */
   void dismiss()
   {
      if (mqgsm != null)
      {
         mqgsm.stopPlease();
         mqgsm.interrupt();
         
         mqgom.closeQAndDisconnect();
      }

      this.setVisible(false);

      // wait for the MQ child thread to end.
      try { Thread.sleep(300); } catch (InterruptedException ie) {}

      this.dispose();
      
      System.exit(0);
   }
}
