package com.capitalware.ufmsm.popups;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.*;
import javax.swing.table.*;

import com.capitalware.ufmsm.handlers.StringUtilities;
import com.capitalware.ufmsm.models.WorkflowModel;
import com.capitalware.ufmsm.objects.CWConstants;
import com.capitalware.ufmsm.renders.WorkflowRender;

/**
 * This class will display the results of the processed actions of a workflow. 
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class WorkflowDialog extends JDialog implements ActionListener, ComponentListener
{
   private JFrame          parent = null;
   private JTable          table = null;
   private WorkflowModel   wfm = null;

   /**
    * Create this dialog with the given parent and title.
    *
    * @param parent window from which this dialog is launched
    * @param title the title for the dialog box window
    */
   public WorkflowDialog(JFrame parent, String title)
   {
      super(parent, title, false);
      this.parent = parent;

      if (title==null)
      {
          setTitle(title);
      }
      if (parent != null)
      {
          setLocationRelativeTo(parent);
      }
      // super calls dialogInit, so we don't need to do it again.
   }

   /**
    * Called by constructors to initialize the dialog.
    */
   protected void dialogInit()
   {
      super.dialogInit();
      addComponentListener(this);

      setSize(CWConstants.WK_MIN_WIDTH, CWConstants.WK_MIN_HEIGHT);     // w * h
      getContentPane().setLayout(new GridBagLayout());

      addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent we)
         {
            dialogCleanUp();
         }
      });
      
      menuBar();
      
      getOwner().setEnabled(false);
   }
   
   /**
    * Add a component to main pane.
    * @param c
    * @param gbc
    * @param x
    * @param y
    * @param w
    * @param h
    */
   private void add(Component c, GridBagConstraints gbc2, int x, int y, int w, int h)
   {
      gbc2.gridx = x;
      gbc2.gridy = y;
      gbc2.gridwidth = w;
      gbc2.gridheight = h;
      getContentPane().add(c, gbc2);
   }
   
   /**
    * Create a menu bar for the application.
    */
   private void menuBar( )
   {
      // create menu - File, Edit, View & Help
      JMenuBar menuBar = new JMenuBar();
      setJMenuBar(menuBar);
      
      JMenu fileMenu = new JMenu("File");
      menuBar.add(fileMenu);

      JMenuItem exitItem = new JMenuItem("Exit");
      exitItem.addActionListener(this);
      fileMenu.add(exitItem);

      JMenu HelpMenu = new JMenu("Help");
      menuBar.add(HelpMenu);

      JMenuItem aboutItem = new JMenuItem("About");
      aboutItem.addActionListener(this);
      HelpMenu.add(aboutItem);
   }

   /**
    * Set the status message data.
    * 
    * @param msgData
    * @return void
    */
   public void setMessageData(byte[] msgData)
   {
      ArrayList al = new ArrayList(); 
      String[] workflowData = new String[0];
      Object[] newCells = null;
      
      try
      {
         int s = 0;
         int e = 0;
         workflowData = StringUtilities.toArrayOfString(msgData);
         
         // skip the 1st one
         for (int i=1; i < workflowData.length; i++)
         {
            newCells = new Object[CWConstants.workflowcolumns];
            
            s = 0;
            e = workflowData[i].indexOf(',');
            
            for (int j=0; (j < CWConstants.workflowcolumns - 1) && (e != -1); j++)
            {
               newCells[j] = workflowData[i].substring(s, e);

               s = e + 1;
               e = workflowData[i].indexOf(',', s);
            }
            
            if ( (e != -1) || (s < workflowData[i].length()) )
               newCells[CWConstants.workflowcolumns - 1] = workflowData[i].substring(s);
            
            al.add(newCells);
         }
         
         wfm = new WorkflowModel(al);

         table = new JTable(wfm);
         table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
         // Not allow the User to Move a Column in a JTable Component
         table.getTableHeader().setReorderingAllowed(false);

         ((DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer()).setHorizontalAlignment(SwingConstants.CENTER);
         
         table.addMouseListener(new MouseAdapter()
         {
            public void mouseClicked(MouseEvent evt)
            {
               // check for double click
               if (evt.getClickCount() < 2) return;

               int rr = table.getSelectedRow();

               if (rr >= 0)
               {
                  ActionDetailsDialog add = new ActionDetailsDialog(WorkflowDialog.this, "Action Details");
                  add.setData(wfm.getRowData(rr));
                  add.show();
               }
            }
         });
         
         JScrollPane jSp   = new JScrollPane(table);

         GridBagConstraints gbc = new GridBagConstraints();
         gbc.anchor = GridBagConstraints.CENTER;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.weightx = 1;
         gbc.weighty = 1;
         gbc.insets = new Insets(0,0,0,0);
         
         add(jSp, gbc, 0, 0, 1, 1);  // c , r, cw, rh
         
         sizeOutColumns( );

         validate();
         repaint();
      }
      catch (IOException e)
      {
         JOptionPane.showMessageDialog( null,
                                        "IOException: " + e.getMessage(),
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
      }
   }

   /**
    * Size out the columns of the JTable.
   */
   private void sizeOutColumns()
   {
      TableColumnModel columnModel = table.getColumnModel();
      TableColumn column;
      
      /*
       * Rssize the columns
       */
      int columns = table.getColumnCount();
      for (int i=0; i < columns; i++)
      {
         column = columnModel.getColumn(i);
         
         WorkflowRender gr = new WorkflowRender();
         if (i < columns - 1)
            gr.setHorizontalAlignment(SwingConstants.CENTER);
         else
            gr.setHorizontalAlignment(SwingConstants.LEFT);
         column.setCellRenderer(gr);

         // Do not set the size for the last column
         if (i < table.getColumnCount() - 1)
         {
            column.setPreferredWidth(CWConstants.workflowcolumnWidth[i]);
            column.setMaxWidth(CWConstants.workflowcolumnWidth[i]);
         }
      }
   }

   /**
    * Process the user actions.
    *
    * @return void
    */
   public void actionPerformed(ActionEvent evt)
   {
      String  arg = evt.getActionCommand();

      if (arg.equals("About"))
      {
         AboutBox about = new AboutBox( parent,
                                        CWConstants.PRODUCT_NAME,
                                        CWConstants.VERSION,
                                        CWConstants.RELEASE_DATE,
                                        CWConstants.COPYRIGHT_DATE,
                                        "gif/ufm32.gif");
         about.show();
      }
      else if (arg.equals("Exit"))
      {
         dialogCleanUp();
      }
      else
      {
         System.out.println( arg + " - NOT defined!!!" );
      }
   }

   /**
    * Time to enable our parent and throw ourselves away.
    */
   private void dialogCleanUp()
   {
      getOwner().setEnabled(true);
      this.hide();
   }


   /**
    *
    * @param e
    */
   public void componentResized(ComponentEvent e)
   {
      int width = getWidth();
      int height = getHeight();

      //we check if either the width
      //or the height are below minimum

      boolean resize = false;

      if (width < CWConstants.WK_MIN_WIDTH)
      {
         resize = true;
         width = CWConstants.WK_MIN_WIDTH;
      }

      if (height < CWConstants.WK_MIN_HEIGHT)
      {
         resize = true;
         height = CWConstants.WK_MIN_HEIGHT;
      }

      if (resize)
      {
         setSize(width, height);
      }
   }

   public void componentMoved(ComponentEvent e)
   {
   }

   public void componentShown(ComponentEvent e)
   {
   }

   public void componentHidden(ComponentEvent e)
   {
   }
}
